#ifndef irkgaussMexh
#define irkgaussMexh

/* Optionsettings */
#define OPT_WARNMISS "OptWarnMiss"
#define OPT_WARNTYPE "OptWarnType"
#define OPT_WARNSIZE "OptWarnSize"

/* Rest */
#define OPT_NSTEP "nstep"
#define OPT_METHOD "method"
#define OPT_FUNCCALLMETHOD "FuncCallMethod"

typedef void (*irkgaussRightSide) (int *n, double *tau,
  double *w, double *f);

typedef void (*irkgaussSaveState) (int *n, int *iter, 
  double *tauOld, double *tau, double *w);

struct ParameterIO 
{ /* parameters for options */
  const mxArray *opt;   /* Options */
  int optCreated;       /* Flag, if options were created by Mex-Interface */
};
typedef struct ParameterIO SParameterIO;

struct ParameterGlobal
{ /* global variables, etc. */
  int d;                /* dimension of the system */
  double *tPointer;     /* Pointer in callers t-Array */
  int funcCallMethod;   /* method, to call matlab: 
                           0: use mexCallMATLAB direct
                           1: use maxCallMATLAB to call feval */
  mxArray *tGrid;       /* return vector for t values */
  mxArray *xGrid;       /* return matrix for states x */
  double *ptGrid;       /* pointer for writing in tGrid */
  double *pxGrid;       /* pointer for writing in xGrid */
};
typedef struct ParameterGlobal SParameterGlobal;

struct ParameterRightSide
{ /* Parameter for right side */
  char *rightSideFcn;   /* Functionname (string) for right side */
  const mxArray *rightSideFcnH; /* function handle or inline-function for
                                   right side */
  mxArray *tArg;        /* for calling f: t */
  mxArray *xArg;        /* for calling f: x */
};
typedef struct ParameterRightSide SParameterRightSide;

struct ParameterIRKGauss 
{ /* parameters for irkgauus */
  double tau,tauEnd;    /* time: start and end */
  double *x;            /* state vector */
  int nstep;            /* number of steps */
  int method;           /* method to use */
  int IOUT;             /* switch for output */
};
typedef struct ParameterIRKGauss SParameterIRKGauss;

#ifdef FORTRANNOUNDER
/* Fotran functions without underscore */
#ifdef FORTRANUPP
/* Fotran functions without underscore  & UPPERCASE letters */
#define IRKGAUSS_ IRKGAUSS
#else
/* Fotran functions without underscore  & lowercase letters */
#define IRKGAUSS_ irkgauss
#endif
#else
/* Fortran functions with underscore */
#ifdef FORTRANUPP
/* Fortran functions with underscore & UPPERCASE letters */
#else
/* Fortran functions with underscore & lowercase letters */
#define IRKGAUSS_ irkgauss_
#endif
#endif

extern void IRK_GAUSS_ (int *n, irkgaussRightSide fcn,
  irkgaussSaveState fcnSave,
  int *nstep, double *tau, double *W, double *tauEnd,
  int *meth, int *iout);

#endif
